/****************************************************************************
Date: 8/4/00
File: ft100.hpp
Description:
	Yaseu FT-100 radio driver

History:
	J version
*****************************************************************************/
#ifndef FT100_HPP
#define FT100_HPP

#include "loctypes.h"
#include "radio.hpp"	// RADIO base class
#include "comdrv.hpp"	// COM port driver class
#include "inttimer.hpp"	// interval timer


//***** Frequency Steps ****************
const LONG FT100_RXSTEPSIZE	= 10;	// 10 Hz receiver tuning step size
const LONG FT100_TXSTEPSIZE	= 100;	// 100 Hz transmitter tuning step size

//***** Message Definitions ************
// All messages to radio are 5 bytes
const INT FT100_TOMESSAGESIZE	= 5;  	// number of bytes in messages to radio
const INT FT100_FROMMESSAGESIZE	= 40;  	// max bytes in messages from radio

// Received message sizes
const INT FT100_READMETERSIZE	= 9;	// "read meter" returns 9 bytes
const INT FT100_STATUSUPDATESIZE= 32;	// "status update" returns 32 bytes

// Sent Message consists of 4 bytes for
// parameters and 1 byte for opcode.
// Opcode is always last.
const INT FT100_MESSAGEPARAMS	= 4;  	// number of params in message to radio
const INT FT100_MESSAGEP1	= 3;	// Param 1 command code goes here

//****** OpCodes ***********************
// Radio command codes
const UCHAR FT100_SPLITMODE	= 0x01;	// split on/off
const UCHAR FT100_VFOMODE	= 0x05;	// A or B
const UCHAR FT100_SETFREQ	= 0x0a;	// set vfo frequency
const UCHAR FT100_SETMODE	= 0x0c; // SSB, CW, etc
const UCHAR FT100_STATUSUPDATE	= 0x10;	// read status
const UCHAR FT100_READMETER	= 0xf7; // read meter (and PTT)

// Radio Sub command codes
const UCHAR FT100_SPLITMODE_OFF	= 0x00;
const UCHAR FT100_SPLITMODE_ON	= 0x01;
const UCHAR FT100_VFOMODE_A	= 0x00;
const UCHAR FT100_VFOMODE_B	= 0x01;

// Mike switch return codes
const UCHAR FT100_PTT		= 0x38;	// Radio does not agree with manual!


//***** COM PORT PARAMETERS ******************************************
const USHORT FT100_TIMEOUT	= 1;	// 1 second response timer
const UINT	FT100_BITRATE	= 4800; // default bit rate
const UINT	FT100_DATABITS	= 8;
const UINT	FT100_STOPBITS	= 2;
const UARTPARITY FT100_PARITY	= UARTPARITY_NONE;

class FT100 : public RADIO {
private:

	//***** Update State **************************************
	// The "Update" routine is a state machine that processes
	// messages from the radio. The state is needed to handle 
	// received messages properly because responses from the
	// FT-100 do not have an opcode.
	enum STATE {
		STATE_STARTPOLL,	// Start info poll cycle
		STATE_READMETER,	// Read Meter (get PTT status)
		STATE_STATUSUPDATE,	// Status update (VFO freq and mode)
		STATE_CHECK,		// Verify PTT  (uses "Read Meter")
		STATE_IDLE		// nothing to do
	};

	INTERVALTIMER	Timer;		// used for COM timeout
	STATE		eState; 	// expected message type from radio
	BOOL		eSplitMode;	// G6LVB 13 APR 2001 only split in XCVR mode
	UINT		unMessageSize;	// expected message size from radio

	BOOL		ePTT;		// Push-to-talk status

	// Rx Params
	LONG		lRxVFOSteps;	// last receiver VFO steps sent to radio
	LONG		lRxVFORead;	// freq read from Rx VFO;
	UCHAR		ucRxBand;	// Frequency band read from Rx VFO
	MODULATION	eRxMode;	// current rx modulation type
	UCHAR		ucRxModeRead;	// rx modulation byte from radio
	BOOL		eUserHasTunedRx;// TRUE if user changed VFO freq
	

	// Tx Params
	LONG		lTxVFOSteps;	// last VFO steps sent to radio
	MODULATION	eTxMode;	// current Tx modulation type
	BOOL		eTxLockout;	// lock out TX updates
	
	// Com port driver
	COMDRV		Comport;
	
	// Message Buffers
	struct {
		UCHAR	aParams[FT100_MESSAGEPARAMS];
		UCHAR	ucOpcode;
	} OutputBuffer;

	UCHAR	aInputBuffer[FT100_FROMMESSAGESIZE];
	UINT	unByteCounter;	// received bytes in input buffer
	

	//******************************************************
	// Utilities
	//******************************************************
	
	// Table to convert RADIO::MODULATION type to FT-100 message param
	static const UCHAR aSetModeTable[];

	// Table to convert FT-100 message param to RADIO::MODULATION type
	static const MODULATION aGetModeTable[];
	
	// Send a message to the radio
	VOID	SendMessage(UCHAR ucOpcode) {
		OutputBuffer.ucOpcode = ucOpcode;
		Comport.Write(&OutputBuffer, FT100_TOMESSAGESIZE);
		Timer.Start(1);
	}

	// Send a message with a P1 SubCmd
	VOID	SendMessage(UCHAR ucOpcode, UCHAR ucSubCmd) {
		OutputBuffer.aParams[FT100_MESSAGEP1] = ucSubCmd;
		SendMessage(ucOpcode);
	}

	// Set radio mode
	VOID	SetMode(MODULATION eMode) {
		SendMessage(FT100_SETMODE, aSetModeTable[eMode]);
	}

	// Set Radio VFO frequency using number of frequency steps
	VOID	SetFrequency(LONG lFreqSteps, LONG lStepSize);

	// Convert Frequency to steps
	LONG	FreqtoSteps(DOUBLE dFrequency, LONG lStepSize) {

		//************************************************
		// WARNING: this code has a workaround for MS C7 bug
		// converting double to long. Do NOT change!
		//************************************************
		volatile ULONG ulFrequency = (ULONG) dFrequency;
		LONG lFrequency = (LONG) ulFrequency;
		return ( (lFrequency) + (lStepSize/2) ) / lStepSize;
	}

	// No copy constructor or assignment allowed
	// Force compile error if attempted
	FT100(const FT100&);
	FT100& operator=(const FT100&);

public:
	// General Functions
	FT100(const RADIOPARAMS& Params);
	BOOL	IsReady();
	VOID	Update();
	~FT100();

	// Tx Functions
	VOID	SetTxFreq(DOUBLE dFrequency);
	VOID	SetTxMode(MODULATION eMode);

	// Rx Functions
	BOOL	HasUserTunedRx() { return eUserHasTunedRx; }
	VOID	RefreshRxParams();
	DOUBLE	GetRxFreq();
	MODULATION GetRxMode() { return eRxMode; }
	VOID	SetRxFreq(DOUBLE dFrequency);
	VOID	SetRxMode(MODULATION eMode);

};
#endif // end ft100.hpp
