/****************************************************************************
Date: 11/11/00
File: ft847a.cpp
Description:
	FT847 radio driver

History:
	12/24/97 Test version
	1/6/99 Version 2, read VFO and mode
	2/14/99 bug fix (ready flag not initialized)
	3/28/99 add XIT
	5/18/99 make frequency step always 10 Hz
	11/11/00 Add "FM repeater" mode for ISS etc.

*****************************************************************************/
#include "ft847.hpp"

// Set Mode Table
// Index into table is MODULATION type, returns FT-847 mode code
const UCHAR FT847::aSetModeTable[] = {
	0x01,	// NA	(USB is default)
	0x00,	// LSB	
	0x01,	// USB
	0x02,	// CW
	0x08,	// FM
	0x04,	// AM
	0x01	// FSK	(no FSK mode, use USB)
};

// Get Mode Table
// index into table is lower 7 bits of FT-847 code, returns MODULATION type
const MODULATION FT847::aGetModeTable[] = {
		MODULATION_LSB, 	// 0
		MODULATION_USB, 	// 1
		MODULATION_CW,	// 2 also 0x82
		MODULATION_CW,	// 3 CW-R also 0x83
		MODULATION_AM,	// 4 also 0x84
		MODULATION_NA,	// 5 invalid code
		MODULATION_NA,	// 6 invalid code
		MODULATION_NA,	// 7 invalid code
		MODULATION_FM	// 8 also 0x88
};


/****************************************************************************
Function: SetFrequency()				PRIVATE
Description: Set radio to this frequency
Parameters:
	Steps = desired frequency in steps
	ucOpcode = opcode to use in message
Returns:
*****************************************************************************/
VOID	FT847::SetFrequency(LONG	lFrequencySteps,
			    UCHAR	ucOpcode) {

	// Convert Steps to frequency
	LONG	lFrequency = lFrequencySteps * FT847_STEPSIZE;

	//********************************************
	// Convert frequency to "Yaesu BCD" and
	// save in message params buffer
	//********************************************
	LONG lDivisor	  = 100000000L; 	// (i.e. 100 MHz)
	LONG lTemp;
	for (SHORT nParam = 0; nParam < FT847_MESSAGEPARAMS; nParam++) {

		// Make upper nibble
		lTemp = lFrequency / lDivisor;
		lFrequency -= (lTemp * lDivisor);
		OutputBuffer.aParams[nParam] = (UCHAR) lTemp << 4;
		lDivisor /= 10;

		// Add in lower nibble
		lTemp = lFrequency / lDivisor;
		lFrequency -= (lTemp * lDivisor);
		OutputBuffer.aParams[nParam] += (UCHAR) lTemp;
		lDivisor /= 10;
	}

	SendMessage(ucOpcode);
	return;
}

/****************************************************************************
Function: ProcessVFO()				PRIVATE
Description: Decode VFO message from radio
	     Received message from radio is in aInputBuffer[]
Return Parameters:
	lFreqSteps = returned frequency steps
	emode	   = returned modulation type

Returns:
*****************************************************************************/
VOID	FT847::ProcessVFO(LONG& lFreqSteps, MODULATION& eMode) {



			// 1st 4 bytes in buffer are freq in "Yaesu BCD"
			// byte 0 = 100MHz and 10MHz
			// byte 1 = 1MHz and 100KHz
			// byte 2 = 10KHz and 1 KHz
			// byte 3 = 100Hz and 10Hz

			LONG lFreq = 0;		// frequency accumulator
			LONG lMultiplier = 100000000L;	// (i.e. 100MHz)

			for ( INT i = 0; i < 4; i++) {

				// Process upper digit
				lFreq += ((aInputBuffer[i] & 0xf0) >> 4) *
								lMultiplier;
				lMultiplier /= 10;

				// Process lower digit
				lFreq += (aInputBuffer[i] & 0x0f) * lMultiplier;
				lMultiplier /= 10;
			}

			// Next, get modulation and convert
			// to RADIO::MODULATION enum type
			// byte 4 = modulation code
			// 00=LSB, 01=USB, 02=CW, 03=CW-R, 04=AM, 08=FM, 
			// 82=CW(N), 83=CW(N)-R, 84=AM(N), 88=FM(N)
		
			eMode = aGetModeTable[(aInputBuffer[4] & 0x7f)];

			// Calculate freq steps
			lFreqSteps = (lFreq + (FT847_STEPSIZE/2)) /
							FT847_STEPSIZE;

			// Return params
			// lFreqSteps = frequency steps
			// eMode = modulation

			return;
}

/****************************************************************************
Function: FT847()
Description: constructor
Parameters:
	RADIOPARAMS structure
Returns:
*****************************************************************************/
FT847::FT847(const RADIOPARAMS& Params) :
	
	// Initialize
	RADIO(),
	eReady(BOOL_TRUE),
	eMessage(FT847_MESSAGE_NONE),
	lRxVFOSteps(0),
	eRxMode(MODULATION_NA),
	eUserHasTunedRx(BOOL_FALSE),
	lTxVFOSteps(0),
	lTxOffset(0),
	lTxXITSteps(0),
	eTxMode(MODULATION_NA),
	unByteCounter(0) {

	//********************************************************
	// Open com port and check communications with radio
	// If fails, return with Operational flag FALSE
	//********************************************************
	RADIO::eOperational = Comport.Open(
					Params.unPort,
					Params.unRate ?
						Params.unRate :
						FT847_BITRATE,
					FT847_DATABITS,
					FT847_PARITY,
					FT847_STOPBITS,
					Params.unIRQ,
					Params.Address);

	if (!RADIO::eOperational) {
		return;		// failed to open
	}

	//**********************************************************
	// Check communications with Radio.
	// Send test message to Radio and start a 1 second timer.
	// If timer expires before we receive test message response,
	// communications has failed.
	//***********************************************************

	SendMessage(FT847_CATON);		// Turn on CAT mode

	// See if radio is talking
	SendMessage(FT847_GETVFO);
	USHORT		unCounter = 0;
	Timer.Start(1);
	while(unCounter < FT847_MESSAGESIZE) { 
		if (Timer.IsExpired()) {
 			// Radio failed to communicate
			RADIO::eOperational = BOOL_FALSE;
			return;
		}
		if (Comport.Getc() != COMDRV_EOF) {
			unCounter++;
		} 
	} // Radio is talking!

// For now, turn off repeater mode
eRepeaterMode = BOOL_FALSE;

	// if satellite (full duplex) mode
	if (Params.eTransceive) {

		eSatMode = BOOL_TRUE;

		// use satellite VFO's
		ucSetRxFreq = FT847_SETRXFREQ;
		ucSetRxMode = FT847_SETRXMODE;
		ucSetTxFreq = FT847_SETTXFREQ;
		ucSetTxMode = FT847_SETTXMODE;
		ucGetRxInfo = FT847_GETRXVFO;

		SendMessage(FT847_SATMODEON);
	}
	else {

		// using radio as either tx or rx but not both
		eSatMode = BOOL_FALSE;

		// use main VFO
		ucSetRxFreq = FT847_SETFREQ;
		ucSetRxMode = FT847_SETMODE;
		ucSetTxFreq = FT847_SETFREQ;
		ucSetTxMode = FT847_SETMODE;
		ucGetRxInfo = FT847_GETVFO;

		SendMessage(FT847_SATMODEOFF);
	}

	return;
}

/****************************************************************************
Function: 	~FT847()
Description:	destructor
Parameters:
Returns:
*****************************************************************************/
FT847::~FT847() {

	// Turn off SAT mode (does nothing if not on)
	SendMessage(FT847_SATMODEOFF);

	// Turn off CAT mode
	SendMessage(FT847_CATOFF);

	// Wait for messages to be sent
	Timer.Start(1);
	while (!Timer.IsExpired()) { }

	// Clean-up
	Comport.Close();
	return;
}

/****************************************************************************
Function: Update()
Description:
		Processes messages received from radio
Parameters:
	
Returns:
*****************************************************************************/
VOID	FT847::Update() {

	// if radio is ready, nothing to do
	if (eReady) {
		return;
	}

	// Check for next message byte received
	INT nByte = Comport.Getc();
	if (nByte == COMDRV_EOF) {	   // nothing received
		if (Timer.IsExpired()) {   // Check for message timeout
			RefreshRxParams(); // if timed out, try again
		}
		return;		// no byte received
	}
	
	// Got a byte so save it
	aInputBuffer[unByteCounter++] = (UCHAR) nByte;

//HERE
	
	if (eMessage == FT847_MESSAGE_TXSTATUS) {
		// End of message
		Timer.Stop();		// stop message timeout timer
		unByteCounter = 0;	// reset byte counter

		// Check PTT bit in response
		// if bit == 0, TX is ON (ePTT is TRUE) 
	     	if ( (nByte && FT847_PTTSTATUS) == 0 ) { 
			// transmitting so quit
			eMessage = FT847_MESSAGE_NONE;
			eReady = BOOL_TRUE;
			return;
		}

		DoNextState();

		// Not transmitting
			
		// If this is the first PTT check,
		// Send: Get MAIN VFO

		// else If this is the second PTT check,
		// save main vfo freq
	}
	else { // this is a "Get VFO" response message
		
		// Save the byte
		aInputBuffer[unByteCounter++] = (UCHAR) nByte;

		// If this is the last byte in message, process it
		if ( unByteCounter == FT847_MESSAGESIZE ) {

			Timer.Stop();		// stop message timeout timer
			unByteCounter = 0;	// reset 

			LONG		lFreqSteps;
			MODULATION	eMode;

			// Decode message
			ProcessVFO(lFreqSteps, eMode);
			switch (eMessage) {
			case FT847_MESSAGE_RXVFO:	// SAT mode
			
				// See if user has manually tuned radio
				// if so, save new freq steps
				if (lFreqSteps != lRxVFOSteps) {
					eUserHasTunedRx = BOOL_TRUE;
					lRxVFOSteps = lFreqSteps;
				}
	
				// Save mode
				eRxMode = eMode;

				if (eSatMode) {
					// send get TxVFO message
					SendMessage(FT847_GETTXVFO);
					eMessage = FT847_MESSAGE_TXVFO;
					Timer.Start(1);
				}
				else {	// done
					eMessage = FT847_MESSAGE_NONE;
					eReady = BOOL_TRUE;
				}
			} 
			else { // FT847_MESSAGE_TXVFO received

				// if user has changed sub band tuning
				// add in new XIT steps
				lTxXITSteps += (lFreqSteps - lTxVFOSteps);
		
				eMessage = FT847_MESSAGE_NONE;
				eReady = BOOL_TRUE;
			}

		} // end of process received message
	}

	// Check for message timeout
	if (Timer.IsExpired()) {

		// if timed out, try again
		RefreshRxParams();
	}

	return;
}

/****************************************************************************
Function: SetTxFreq()
Description:
	Set transmit frequency if different from current frequency
Parameters:
	Frequency
Returns:
*****************************************************************************/
VOID	FT847::SetTxFreq(DOUBLE dFrequency) {

	// Calculate new frequency steps 
	LONG lNewTxVFOSteps = FreqtoSteps(dFrequency);

	// Adjust for sub band tuning (XIT)
	lNewTxVFOSteps += lTxXITSteps;

	// Check to see if freq steps has changed
	// if so, update VFO steps and send new freq to radio
	if (lTxVFOSteps != lNewTxVFOSteps) {
		lTxVFOSteps = lNewTxVFOSteps;
		SetFrequency(lNewTxVFOSteps, ucSetTxFreq);
	}
	return;
}

/****************************************************************************
Function: SetTxMode()
Description:
	Set transmitter modulation mode type
Parameters:
	Mode
Returns:
*****************************************************************************/
VOID	FT847::SetTxMode(MODULATION eMode) {

	// Send message if different from current tx mode
	if (eTxMode != eMode) {
		SetMode(ucSetTxMode, eMode); // send message to radio
		eTxMode = eMode;	// Save new tx mode
	}
	return;
}

/****************************************************************************
Function: RefreshRxParams()
Description:
Parameters:
Returns:
*****************************************************************************/
VOID	FT847::RefreshRxParams() {

	eReady = BOOL_FALSE;
	SendMessage(ucGetRxInfo);
	eMessage = FT847_MESSAGE_RXVFO;	// sending get rx vfo message
	Timer.Start(1);			// 1 second timeout timer
	return;
}

/****************************************************************************
Function: GetRxFreq()
Description:
	Gets Receive (VFO) frequency
Parameters:
Returns:
	Frequency
*****************************************************************************/
DOUBLE	FT847::GetRxFreq() {
	eUserHasTunedRx = BOOL_FALSE;
	return (DOUBLE) (lRxVFOSteps * FT847_STEPSIZE);
}

/****************************************************************************
Function: SetRxFreq()
Description:
	Set Receive frequency
Parameters:
	Frequency
Returns:
*****************************************************************************/
VOID	FT847::SetRxFreq(DOUBLE dFrequency) {

	eUserHasTunedRx = BOOL_FALSE;
	LONG lNewRxVFOSteps = FreqtoSteps(dFrequency);
	if (lRxVFOSteps != lNewRxVFOSteps) {
		lRxVFOSteps = lNewRxVFOSteps;
		SetFrequency(lNewRxVFOSteps, ucSetRxFreq);
	}

	return;
}

/****************************************************************************
Function: SetRxMode()
Description:
	Set receive modulation mode type.
Parameters:
	Mode
Returns:
*****************************************************************************/
VOID	FT847::SetRxMode(MODULATION eMode) {

	// Send message if new mode is different from current mode
	if (eRxMode != eMode) {
		SetMode(ucSetRxMode, eMode); // Send message to radio
		eRxMode = eMode; 	// Save new Rx mode
	}
	return;
}

// end FT-847.cpp
