/****************************************************************************
Date: 8/18/2001
File: icom.cpp
Description:
	ICOM Radio Driver for IC-821
History:
	8/18/2001 radio.hpp parameter name change (unParam1)
*****************************************************************************/
#include "icom.hpp"


/****************************************************************************
Function: Constructor
Description:
Parameters:
Returns:
*****************************************************************************/
ICOM::ICOM(const RADIOPARAMS& Params) :

	RADIO(),			// Base class
	Msg(),				// Msg Handler
	eReady(BOOL_FALSE),
	eFullDuplex(Params.eTransceive),
	ucCurrentBand(0),		// This means "unknown"
	eState(ICOM_STATE_IDLE),
	lRxVFOSteps(0),
	eRxMode(MODULATION_NA),
	eUserHasTunedRx(BOOL_FALSE),
	lTxVFOSteps(0),
	eTxMode(MODULATION_NA) {

	RADIO::eOperational = Msg.Open( Params.unParam1,	// CI-V Address
					Params.unPort,		// COM port
					Params.unRate,		// Bit rate
					Params.unIRQ,		// HW - IRQ
					Params.Address);	// HW IO address

	if (!RADIO::eOperational) {
		return;		// Msg handler failed to open COM port
	}

	// initialize Radio
	// turn off full duplex mode
	Msg.SendCmd(ICOMMSG_SPLITMODE, ICOMMSG_CANCELDUPLEX);
	eState = ICOM_STATE_IDLE;
	return;
}

/****************************************************************************
Function: Update() 
Description:
	This is the state machine that processes all
	radio messages. It runs every clock tick.
Parameters:
Returns:
*****************************************************************************/
VOID	ICOM::Update() {

	// If READY flag is set, nothing to do
	if (eReady) {
		return;
	}

	// Check for a received message
	if (!Msg.IsReady()) {
		return;				// nothing to do
	}

	// Process Message based on command type
	switch(Msg.GetCmd()) {

	case ICOMMSG_READFREQ:
	    {
		LONG lNewRxVFOSteps = GetSteps(Msg.GetFrequency(), eRxMode);
		if (lNewRxVFOSteps != lRxVFOSteps) {
			eUserHasTunedRx = BOOL_TRUE;
			lRxVFOSteps = lNewRxVFOSteps;
		}
		eState = ICOM_STATE_IDLE; // done refreshing rx params
		eReady = BOOL_TRUE;
	    }
	    break;	

	case ICOMMSG_READMODE:
		eRxMode = Msg.GetMode();
		// Got the RX mode, now get the RX freq
		Msg.SendCmd(ICOMMSG_READFREQ);
		break;

	case ICOMMSG_OK:
		switch(eState) {
		case ICOM_STATE_SETTXFREQ:
			Msg.SendWriteFrequency(lTxVFOSteps * 
						GetStepSize(eTxMode));
			eState = ICOM_STATE_WAITOK;
			break;
		case ICOM_STATE_SETTXMODE:
			Msg.SendWriteMode(eTxMode);
			eState = ICOM_STATE_WAITOK;
			break;
		case ICOM_STATE_SETRXFREQ:
			Msg.SendWriteFrequency(lRxVFOSteps *
						GetStepSize(eRxMode));
			eState = ICOM_STATE_WAITOK;
			break;
		case ICOM_STATE_SETRXMODE:
			Msg.SendWriteMode(eRxMode);
			eState = ICOM_STATE_WAITOK;
			break;
		case ICOM_STATE_GETRXPARAMS:
			Msg.SendCmd(ICOMMSG_READMODE);
			break;
		case ICOM_STATE_WAITOK:
			eState = ICOM_STATE_IDLE;
			eReady = BOOL_TRUE;
			break;
		default: // shouldn't be here
			eState = ICOM_STATE_IDLE;
			eReady = BOOL_TRUE;
			break;
	    	} // end switch() ICOMMSG_OK processing
		break;
	default: // no such message, ignore
		break;
	} // end msg processing

	return;
}

/****************************************************************************
Function: Destructor
Description:
Parameters:
Returns:
*****************************************************************************/
ICOM::~ICOM() {
	Msg.Close();
}


/****************************************************************************
Function: SetTxFreq()
Description:
	Sets Transmitter frequency
Parameters:
	DOUBLE 	dFrequency
Returns:
*****************************************************************************/
VOID	ICOM::SetTxFreq(DOUBLE dFrequency) {

	//***** Get new TX VFO steps *****
	LONG lFrequency = FreqtoLong(dFrequency);
	LONG lNewTxVFOSteps = GetSteps(lFrequency, eTxMode);
	if (lNewTxVFOSteps == lTxVFOSteps) {
		return;				// no change
	}
	lTxVFOSteps = lNewTxVFOSteps;		// Save New Tx Freq Steps

	if ( IsCurrentBand(ICOMMSG_MAIN) ) {
		Msg.SendWriteFrequency(lFrequency);	
		eState = ICOM_STATE_WAITOK;
	}
	else {
		eState = ICOM_STATE_SETTXFREQ;
	}

	eReady = BOOL_FALSE;
	return;
}


/****************************************************************************
Function: SetTxMode()
Description: Set transmit modulation type
Parameters:
	MODULATION eMode
Returns:
*****************************************************************************/
VOID	ICOM::SetTxMode(MODULATION eMode) {

	if (eMode == eTxMode) { 		// if Mode has changed...
		return;
	}
	eTxMode = eMode; 		// Save New Tx Mode

	if ( IsCurrentBand(ICOMMSG_MAIN) ) {
		Msg.SendWriteMode(eMode);	
		eState = ICOM_STATE_WAITOK;
	}
	else {
		eState = ICOM_STATE_SETTXMODE;
	}

	eReady = BOOL_FALSE;
	return;
}

/****************************************************************************
Function: RefreshRxParams()
Description:
	Poll radio to get receiver frequency and mode
Parameters:
Returns:
*****************************************************************************/
VOID	ICOM::RefreshRxParams() {

	if ( IsCurrentBand(ICOMMSG_SUB) ) {
		Msg.SendCmd(ICOMMSG_READMODE); 
	}

	eState = ICOM_STATE_GETRXPARAMS;
	eReady = BOOL_FALSE;
	return;
}

/****************************************************************************
Function: GetRxFreq()
Description:
	Returns current receiver frequency from previously
	refreshed rx params
Parameters:
Returns:
	DOUBLE	dFrequency
*****************************************************************************/
DOUBLE	ICOM::GetRxFreq() {

	// Set flag to show that user has not tuned from this returned freq
	eUserHasTunedRx = BOOL_FALSE;

	// Convert RX VFO Frequency Steps to Double
	LONG lFrequency = lRxVFOSteps * GetStepSize(eRxMode);
	return (DOUBLE) lFrequency;
}


/****************************************************************************
Function: SetRxFreq()
Description:
	Sets Receiver frequency
Parameters:
	DOUBLE 	dFrequency
Returns:
*****************************************************************************/
VOID	ICOM::SetRxFreq(DOUBLE dFrequency) {

	//***** clear user has tuned flag ****
	eUserHasTunedRx = BOOL_FALSE;

	//***** Get new RX VFO steps *****
	LONG lFrequency = FreqtoLong(dFrequency);
	LONG lNewRxVFOSteps = GetSteps(lFrequency, eRxMode);
	if (lNewRxVFOSteps == lRxVFOSteps) {
		return;				// unchanged
	}
	lRxVFOSteps = lNewRxVFOSteps; 	// Save New Rx Frequency Steps

	if ( IsCurrentBand(ICOMMSG_SUB) ) {
		Msg.SendWriteFrequency(lFrequency);	
		eState = ICOM_STATE_WAITOK;
	}
	else {
		eState = ICOM_STATE_SETRXFREQ;
	}

	eReady = BOOL_FALSE;
	return;
}

/****************************************************************************
Function: SetRxMode()
Description:
	Sets receiver modulation
Parameters:
	MODULATION eMode
Returns:
*****************************************************************************/
VOID	ICOM::SetRxMode(MODULATION eMode) {

	if (eMode == eRxMode) {
		return;				// unchanged
	}
	eRxMode = eMode;			// save new mode


	if ( IsCurrentBand(ICOMMSG_SUB) ) {
		Msg.SendWriteMode(eMode);	
		eState = ICOM_STATE_WAITOK;
	}
	else {
		eState = ICOM_STATE_SETRXMODE;
	}

	eReady = BOOL_FALSE;
	return;
}
// end icom.cpp
