/****************************************************************************
Date: 1/21/98
File: itune.cpp

Description:
	InstantTune command line control program 
	Controls TSR through command interrupt
*****************************************************************************/
#include "itune.hpp"

#pragma check_stack(off)
#pragma check_pointer(off)

//***** CONFIG ERROR CHECKER ************************************************
// Error string table - must be in same order as CONFIG_ERROR enum
const CHAR* const ITUNECONFIGCHECK::aErrorString[] = {
	"couldn't open config file",
	"already have uplink or translation",
	"already have beacon/uplink",
	"already have radio for this function",
	"unknown radio type",
	"already have frequency converter",
	"com port out of range",
	"unknown command",
	"software bug detected!"
};

VOID ITUNECONFIGCHECK::Error(CONFIG_ERROR eErrorType) {
	eError = BOOL_TRUE;

	cout << endl;

	if (eErrorType != CONFIG_ERROR_CFGFILEOPEN) {
		cout << "##### Error at line number " << Parser.GetLineNumber();
		cout << " #####" << endl;
		cout << "SATELLITE: " << GetSatelliteName() << endl;
		cout << "MODE: " << GetConfigID() << endl;
		cout << "BAD LINE: " << Parser.GetLine() << endl;
	}

	cout << "PROBLEM: " << aErrorString[eErrorType] << endl;
	return;
}

BOOL	ITUNECONFIGCHECK::ErrorScan() {

	// To scan all of config file for errors, get the
	// config for an invalid sat name.
	// this forces parse of entire config file
	// Comment: this is not the best way to do this because it
	// is dependent on the implementation of the CONFIG class
	//  It was easy though and I'll  fix it if I have a chance.
	
	GetConfig("INVALID SATELLITE NAME");
	return AnyErrors();
}

//***** CONFIG SELECTOR ****************************************************
BOOL 	ITUNECONFIGSELECT::SetMode(const CHAR* pszSatName,
				   const CHAR* pszConfigID) {

	// Try to create a file "itune.tmp"
	FILE* pTempFile = fopen(ITUNE_TEMPFILE, "w");
	if (!pTempFile) {
		// if can't print error message and exit
		cout << "Error, can't create a temp file" << endl;
		return BOOL_FALSE;
	}

	// Try to open parser with config select file
	BOOL	eSatFound = BOOL_FALSE;
	if ( Parser.Open(CONFIG_SELECTFILE) ) {

		// Go through entire select file
		// GetToken() returns NULL when EOF
		const CHAR* pszTempSatName;
		while( pszTempSatName = Parser.GetToken() ) {

			// save sat name in temp file
			fputs(pszTempSatName, pTempFile);

			// Add white space delimeter between name and mode
			fputs(" ", pTempFile);

			// Get configID
			// If sat names match overwrite configID
			// Save in temp file
			const CHAR* pszTempConfigID = Parser.GetString();
			if ( !stricmp(pszTempSatName, pszSatName) ) {
				eSatFound = BOOL_TRUE;	// Satellite FOUND!
				pszTempConfigID = pszConfigID;
			}
			fputs(pszTempConfigID, pTempFile);

			// Write newline 
			fputs("\n", pTempFile);
		} // end while()
		// close parser and delete old select file
		Parser.Close();
		remove(CONFIG_SELECTFILE);
	}

	// If sat was not found in select file, append it to temp file
	if (!eSatFound) {
		fputs(pszSatName, pTempFile);
		fputs(" ", pTempFile);
		fputs(pszConfigID, pTempFile);
		fputs("\n", pTempFile);
	}

	fclose(pTempFile);

	// Now rename temp file to be new select file
	rename(ITUNE_TEMPFILE, CONFIG_SELECTFILE);

	return BOOL_TRUE;
}


//***** InstantTune Control Program *****************************************
main(INT nArgCount, CHAR* ArgVector[]) {


	REGS	InRegs, OutRegs;
	SREGS	SRegs;

	if (nArgCount < 2) {
		cout << ITUNE_OPTIONS << endl;
		exit(0);
	}

	//***** Get command interrupt number **********************
	// First check environment variables for
	// overrides, otherwise use defaults
	UINT	unCmdInterrupt;
	CHAR*	pszCmdInterrupt = getenv(CMD_VAR_RADIODRV);
	if (pszCmdInterrupt) {
		unCmdInterrupt = (UINT) strtoul(pszCmdInterrupt, NULL, 16);
	}
	else {
		unCmdInterrupt = CMD_INT_RADIODRV;
	}

	//***** Commands that don't require TSR to be loaded ***********
	switch (*ArgVector[1]) {
	case ITUNE_LOAD:
	    	switch( spawnl(P_WAIT, "itunetsr.exe", NULL) ) {

	    	case CMD_RET_SUCCESS:
			cout << "InstantTune TSR Loaded" << endl;
			break;

	    	case CMD_RET_VECTORINUSE:
			if ( IsTSRLoaded(unCmdInterrupt,
					 CMD_SIG_OFFSET,
					 CMD_SIG_RADIODRV) ) {
			     	cout << "InstantTune TSR already loaded";
				cout << endl;
			}
			else {
				cout << "Cannot load TSR, INT ";
				cout << unCmdInterrupt << "(dec)";
				cout << " already in use" << endl;
			}
			break;

		case CMD_RET_NOORBITDRV:
			cout << "InstantTune TSR not loaded, no OrbitDRV";
			cout << endl;
			break;

	    	default: // spawn failed 
			perror("FAIL");
			break;
	    	} // end spawnl() switch()

		return 0;
	    	break;

	case ITUNE_VERIFY:
		{
		    cout << "Verifying configuration file . . ." << endl;
		    ITUNECONFIGCHECK	ConfigChecker;
	            
		    if ( !ConfigChecker.ErrorScan() ) {
		    	cout << "No Errors Found" << endl;
		    }
		}

		return 0;
		break;

	case ITUNE_SETMODE:	// set Mode for this satellite
		if (nArgCount < 3) {
			cout << "error, need a satellite name" << endl;
		}
		else {
			CHAR*	pszMode;
			if ( nArgCount < 4 ) {
				pszMode = "";
			}
			else {
				pszMode = ArgVector[3];
			}

			ITUNECONFIGSELECT Config;
			if ( Config.SetMode(ArgVector[2], pszMode) ) {
				cout << "Satellite " << ArgVector[2]; 
				cout << " set to use ";
				if (!(*pszMode)) {
					cout << "default mode";
				}
				else {
					cout << "mode " << pszMode;
				}
				cout << endl;
			}
		} //end else
		return 0;
		break;

	default:
		break;
	} // end switch() for commands not requiring TSR to be loaded
	
	//***** Commands that require TSR to be loaded ********************
	// Make sure TSR is loaded
	if ( !IsTSRLoaded(unCmdInterrupt,
			  CMD_SIG_OFFSET,
			  CMD_SIG_RADIODRV) ) {

		cout << "InstantTune TSR is not loaded" << endl;
		exit(0);
	}

	switch (*ArgVector[1]) {
	case ITUNE_SETSATELLITE: // set satellite configuration
		{
			CHAR FAR* lpszSatName = ArgVector[2];
			InRegs.h.ah = CMD_RADIODRV_SETCONFIG;
			InRegs.x.dx = FP_OFF(lpszSatName);
			SRegs.ds    = FP_SEG(lpszSatName);
			int86x(unCmdInterrupt, &InRegs, &OutRegs, &SRegs);
			if (OutRegs.x.ax == BOOL_TRUE) {
				cout << "Satellite Set to " << ArgVector[2];
				cout << endl;
			}
			else {
				cout << "Set Satellite Failed" << endl;
			}
	    	}
	    	break;

	case ITUNE_TUNE: // simulate tune 
		{
			DOUBLE		dVelocity;
			DOUBLE FAR*	lpVelocity = &dVelocity;
			if (nArgCount > 2) {
				dVelocity = atof(ArgVector[2]);
			}
			else {
				dVelocity = 0.0;
			}
			InRegs.h.ah	= CMD_RADIODRV_SETVELOCITY;
			SRegs.ds	= FP_SEG(lpVelocity);
			InRegs.x.dx	= FP_OFF(lpVelocity);
			int86x(unCmdInterrupt, &InRegs, &OutRegs, &SRegs);
			cout << "tune using velocity = " << dVelocity << endl;
		}
		break;

	case ITUNE_HALT:
		InRegs.h.ah	= CMD_RADIODRV_STOP;
		int86x(unCmdInterrupt, &InRegs, &OutRegs, &SRegs);
		cout << "Tuning halted" << endl;
		
		break;

	case ITUNE_CHECK:	// check status
		InRegs.h.ah	= CMD_RADIODRV_GETSTATUS;
		int86x(unCmdInterrupt, &InRegs, &OutRegs, &SRegs);
		if ( OutRegs.x.ax ) {
			cout << "Tuning enabled";
		}
	 	else {
			cout << "Tuning disabled, invalid or missing"
				" configuration";
		}
		cout << endl;
		break;

	case ITUNE_DEBUG:
	    {
		InRegs.h.ah	= CMD_RADIODRV_DEBUG;
		int86x(unCmdInterrupt, &InRegs, &OutRegs, &SRegs);
	    }
	    break;

	default:
		// Did not find this command
		cout << "Huh?" << endl;
		break;

	} // end switch(), commands that require TSR to be loaded

	return 0;
}
