/****************************************************************************
Date: 11/23/97
File: kenwood.cpp
Description:
	Basic KENWOOD radio driver
	Controls radios TS450, TS690, TS850, TS950

History:
 3/18/97 - EXPERIMENTAL Floating Point overflow work around

*****************************************************************************/
#include "kenwood.hpp"

// Mode Tables
// Index into table is kenwood mode code, returns MODULATION value
const MODULATION KENWOOD::aGetModeTable[] = {
	MODULATION_NA,	// 0 = invalid
	MODULATION_LSB,	// 1
	MODULATION_USB,	// 2
	MODULATION_CW,	// 3
	MODULATION_FM,	// 4
	MODULATION_AM,	// 5
	MODULATION_FSK,	// 6
	MODULATION_CW,	// 7 = CW reverse
	MODULATION_NA,	// 8 = invalid
	MODULATION_FSK	// 9 = FSK reverse
};

// Index into table is MODULATION type, returns kenwood mode code
const CHAR KENWOOD::aSetModeTable[] = {
	'0',	// NA
	'1',	// LSB
	'2',	// USB
	'3',	// CW
	'4',	// FM
	'5',	// AM
	'6'	// FSK
};

/****************************************************************************
Function: SetFrequency()				PRIVATE
Description:
	Calculates difference between new frequency and current setting.
	If difference is bigger than step size, commands radio to change
	frequency using mic buttons or direct VFO setting and updates
	current frequency step count.

Parameters:
	DOUBLE	dNewFrequency,	// desired new frequency
	LONG	lStepSize,	// frequency step size
	LONG*	plSteps,	// pointer to current step count
	const CHAR* pszSetVFO	// VFO command string

Returns:
*****************************************************************************/
VOID	KENWOOD::SetFrequency(
			DOUBLE	dNewFrequency,
			LONG	lStepSize,
			LONG*	plSteps,
			const CHAR* pszSetVFOCmd) {

	//***********************************************************
	// Check if new frequency is different than current one
	// Convert new frequency to steps and compare with old steps
	// Must be a new step to be different
	//***********************************************************

	volatile ULONG ulFrequency = (ULONG) dNewFrequency;
	LONG lNewSteps = FreqtoSteps( ((LONG) ulFrequency), lStepSize );
	LONG lDiffSteps = lNewSteps - *plSteps;

	if (lDiffSteps == 0) {
		return;		// no difference, do nothing
	}

	// New Frequency is different, Save new step count
	*plSteps = lNewSteps; // Save new steps
	
	//********************************************************
	// If using transceiver mode or difference in steps is too
	// much for mic button adjust, use direct set VFO	 
	// command. Otherwise can use mic button commands
	//********************************************************

	if ( eXCVR || (labs(lDiffSteps) > KENWOOD_BUTTONLIMIT) ) {

		//******************************************************
		// Create "Set VFO" message and Send it out.
		// Put "set VFO" opcode in Buffer. Then convert
		// frequency to a string and put in output buffer.
		// Finally, send message to Serial Link
		//*******************************************************

		CHAR*	pBuffer = aOutputBuffer;
		strcpy(pBuffer, pszSetVFOCmd);
		pBuffer += KENWOOD_OPCODESIZE;

		//************************************************************
		// Maximum frequency that LONG can support is 2GHz
		// This is 10 digits but kenwood format is 11 digits so
		// first write leading zero then iterate over remaining digits
		// converting LONG to a character string. Can't use ltoa()
		// because we need leading zeros.
		//*************************************************************

		*pBuffer++ = '0';	// write leading 0 (10 GHz)

		LONG lFrequency = lNewSteps * lStepSize; // steps to freq
		LONG lDivisor = KENWOOD_1GIGAHERTZ;
		for (INT i=0; i < 10; i++) {
			LONG lDigit = lFrequency / lDivisor; 
			*pBuffer++ = (CHAR) (lDigit + '0');
			lFrequency -= (lDigit * lDivisor);
			lDivisor /= 10;	
		}
		*pBuffer = 0; 	// add nul termination

		SerialLink.Send(aOutputBuffer);
	}
	else {
	
		//********************************************************
		// Use Mic Button commands to adjust frequency.
		// If diff is + use UP command, otherwise use DOWN command.
		// Send one button message per step
		//********************************************************
		const CHAR*	pButtonMessage;
	
		if (lDiffSteps < 0) {
			pButtonMessage = KENWOOD_MICBUTTONDOWN;
		}
		else {
			pButtonMessage = KENWOOD_MICBUTTONUP;
		}

		for (INT i=0; i < labs(lDiffSteps); i++) {
			SerialLink.Send(pButtonMessage, KENWOOD_OPCODESIZE);
		}
	}

	return;
}

/****************************************************************************
Function: SetMode()					PRIVATE
Description:
	Sends radio the "Set Mode" command to set the modulation type
Parameters:
	Mode
Returns:
*****************************************************************************/
VOID	KENWOOD::SetMode(MODULATION eMode) {

	//********************************************
	// Copy "set mode" opcode to output buffer.
	// Set the kenwood mode code for the desired
	// mode in the output buffer. Then send it out.
	//*********************************************
	strcpy(aOutputBuffer, KENWOOD_MODE);
	aOutputBuffer[KENWOOD_OPCODESIZE] = aSetModeTable[eMode];
	SerialLink.Send(aOutputBuffer, KENWOOD_OPCODESIZE + 1);

	return;
}

/*****************************************************************************
Function: GetStepSize()					PRIVATE
Description:
	Gets frequency step size for this modulation mode
Parameters:
	Mode
Returns:
	Step size
*****************************************************************************/
LONG	KENWOOD::GetStepSize(MODULATION eMode) {

	LONG lStep;
	if (eXCVR) {
		lStep = KENWOOD_STEP_XCVR;
	}
	else {
		if ( (eMode == MODULATION_FM) ||
		     (eMode == MODULATION_AM)  ) {

			lStep = KENWOOD_STEP_AMFM;
		}
		else {
			lStep = KENWOOD_STEP_DEFAULT;
		}
	}
	return lStep;
}


/****************************************************************************
Function: KENWOOD()
Description: constructor
Parameters:
	RADIOPARAMS structure
Returns:
*****************************************************************************/
KENWOOD::KENWOOD(const RADIOPARAMS& Params) :
	
	// Initialize
	RADIO(),
	eReady(BOOL_TRUE),
	eXCVR(Params.eTransceive),

	lRxStepSize(KENWOOD_STEP_DEFAULT),
	lRxSteps(0),
	eRxMode(MODULATION_NA),
	eUserHasTunedRx(BOOL_FALSE),

	lTxStepSize(KENWOOD_STEP_DEFAULT),
	lTxSteps(0),
	eTxMode(MODULATION_NA),

	SerialLink(	(CHAR*) aInputBuffer,
			KENWOOD_MAXMESSAGE,
			KENWOOD_ENDOFMESSAGE,
			KENWOOD_TIMEOUT),

	pszSetRxFreq(KENWOOD_VFOA) { 	// Use VFO A for Rx always


	//********************************************************
	// Open Serial Link and check communications with radio
	// If fails, return with Operational flag FALSE
	//********************************************************
	if (SerialLink.Open(
			Params.unPort,
			KENWOOD_BITRATE,
			KENWOOD_DATABITS,
			KENWOOD_PARITY,
			KENWOOD_STOPBITS,
			Params.unIRQ,
			Params.Address) == BOOL_FALSE) {

		RADIO::eOperational = BOOL_FALSE;
		return;					// failed
	}

	// Check communications
	SerialLink.Send(KENWOOD_TESTMESSAGE);// Send test msg, wait for response
	do {
		if ( SerialLink.TimeOut() ) {
			SerialLink.Close(); 		// failed
			RADIO::eOperational = BOOL_FALSE;
			return;
		}
	} while(SerialLink.Receive() == 0);

	//****** Set TX and RX VFO's ****************************************
	// Tx VFO
	const CHAR* pszTxVFO;

	if ( eXCVR ) {	// if using this as a XCVR

		// use VFO B for TX
		pszSetTxFreq = KENWOOD_VFOB;	// use VFO B to set frequncy
		pszTxVFO = KENWOOD_SETTXVFOB;	// send cmd to use VFO B for Tx

		// Override default frequency step
		lRxStepSize = KENWOOD_STEP_XCVR;
		lTxStepSize = KENWOOD_STEP_XCVR;
	} 
	else {
		// Use VFO A for TX
		pszSetTxFreq = KENWOOD_VFOA;	// use VFO A to set freq
		pszTxVFO = KENWOOD_SETTXVFOA;	// send cmd to use VFO A for Tx
	}

	// Send setup messages to radio
	SerialLink.Send(KENWOOD_SETRXVFOA);	// Set Receiver to VFO A
	SerialLink.Send(pszTxVFO);		// Set Transmitter to Tx VFO
	SerialLink.Send(KENWOOD_FINEOFF);

	// All OK!
}

/****************************************************************************
Function: 	~KENWOOD()
Description:	destructor
Parameters:
Returns:
*****************************************************************************/
KENWOOD::~KENWOOD() {
	SerialLink.Close();
}

/****************************************************************************
Function: Update()
Description:
	This function implements the processing for getting the
	receiver status. Processing is started with the RefreshRxParams()
	function which send a get information (i.e. IF) command to the radio
	and clears the Ready flag. When the IF response is received, we
	process the response and then send the get VFO A frequency command
	to the Radio. When we receive this response, we process this message
	and set the Ready flag TRUE.

	The information that we get from the receiver is the current
	modulation mode and the current frequency. 

Parameters:
Returns:
*****************************************************************************/
VOID	KENWOOD::Update() {

	//***** If Ready flag is set, all done already ****************
    	if (eReady) {
		return;
    	}

	//**************************************************************
	// Check serial link for a received message
	// Value returned is byte count of message or 0 if not available
	// Message is returned in InputBuffer
	// If Message, received, decode and process it
	//*************************************************************

	UINT	unCharCount = SerialLink.Receive();

	if ( unCharCount != 0 ) {

		LONG lNewSteps;	// Temporary
		CHAR*	pMsg = aInputBuffer;

		INT nOpCode = ( (INT) pMsg[0] + pMsg[1] );

		switch(nOpCode) {

		case KENWOOD_MSGIF_CODE	:	// Information response

			//**************************************************
			// Information response
			//
			// First need to check if currently in RX mode,
			// then if so, get parameter from message
			// and convert to MODULATION type via table.
			//
			// If mode is AM or FM, kenwwod automatically
			// sets MIC Button step size to 100 Hz. If we are not
			// in XCVR mode, we need to update our Rx frequency
			// step size so we calculate mic button frwquency
			// adjust correctly for current mode 
			//*************************************************

			if ( pMsg[KENWOOD_MSGIF_TXRXOFFSET] == 
						KENWOOD_MSGIF_RX) {

				eRxMode = aGetModeTable[
					(pMsg[KENWOOD_MSGIF_MODEOFFSET]-'0')];

				lRxStepSize = GetStepSize(eRxMode);
			}


			//***** Now get VFO A (RX) frequency ****************
			SerialLink.Send(KENWOOD_VFOA);
			break;

		case KENWOOD_MSGFA_CODE	: // frequency of VFO A (receiver)
			//**********************************************
			// Convert frequency to steps and save.
			// Check for manual tuning by user
			//**********************************************

			lNewSteps = FreqtoSteps( 
					atol(&pMsg[KENWOOD_OPCODESIZE]),
					lRxStepSize);
	
			if (lNewSteps != lRxSteps) {
				eUserHasTunedRx = BOOL_TRUE;
				lRxSteps = lNewSteps;
			}
	
			eReady = BOOL_TRUE; // All done, set ready flag
	    		break;

		default:
			// unexpected message
			break;

		} // end switch()


	} // end if()

	//***** Check for serial link timeout ***********************
	if (SerialLink.TimeOut() ) {
		SerialLink.Reset();	// clear buffers
		RefreshRxParams();	// try again
	}

    	return;
}

/****************************************************************************
Function: SetTxFreq()
Description:
	Sets transmitter to new frequency
Parameters:
	Frequency
Returns:
*****************************************************************************/
VOID	KENWOOD::SetTxFreq(DOUBLE dNewFrequency) {

	SetFrequency(	dNewFrequency,
			lTxStepSize,
			&lTxSteps,
			pszSetTxFreq);
}

/****************************************************************************
Function: SetTxMode()
Description:
	Set transmitter Mode type
Parameters:
	Mode
Returns:
*****************************************************************************/
VOID	KENWOOD::SetTxMode(MODULATION eMode) {
	
	// Check for changed and valid modulation mode
	if ( (eTxMode == eMode) ||
	     (eMode == MODULATION_NA) ) {
		return;				// do nothing
	}

	// Save New mode
	eTxMode = eMode;

	// if XCVR mode, set rx temporarily to vfo B
	if (eXCVR) {
		SerialLink.Send(KENWOOD_SETRXVFOB);
	}
	
	// Send command to radio
	SetMode(eMode);

	// if XCVR set rx back to vfo A, tx back to vfo B
	if (eXCVR) {
		SerialLink.Send(KENWOOD_SETRXVFOA);
		SerialLink.Send(KENWOOD_SETTXVFOB);
	}

	// Adjust Tx Step
	lTxStepSize = GetStepSize(eMode);

	return;
}

/****************************************************************************
Function: RefreshRxParams()
Description:
	Initiate getting radio info (Rx Freq, Rx Modulation)
Parameters:
Returns:
*****************************************************************************/
VOID	KENWOOD::RefreshRxParams() {

	eReady = BOOL_FALSE;
	SerialLink.Send(KENWOOD_INFO);  	// Read info
	return;
}

/****************************************************************************
Function: GetRxFreq()
Description:
	Gets current rx frequency
Parameters:
Returns:
	receiver frequency
*****************************************************************************/
DOUBLE	KENWOOD::GetRxFreq() {
	eUserHasTunedRx = BOOL_FALSE;
	return ((FLOAT) lRxSteps) * ((FLOAT) lRxStepSize);
}

/****************************************************************************
Function: SetRxFreq()
Description:
	Sets Receiver to new frequency
Parameters:
	Frequency
Returns:
*****************************************************************************/
VOID	KENWOOD::SetRxFreq(DOUBLE dFrequency) {

	eUserHasTunedRx = BOOL_FALSE;
	SetFrequency(	dFrequency,
			lRxStepSize,
			&lRxSteps,
			pszSetRxFreq);
}

/****************************************************************************
Function: SetRxMode()
Description:
	Set receive modulation mode type
Parameters:
	Mode
Returns:
*****************************************************************************/
VOID	KENWOOD::SetRxMode(MODULATION eMode) {
	
	// Check for changed and valid modulation mode
	if ( (eRxMode == eMode) ||
	     (eMode == MODULATION_NA) ) {
		return;				// do nothing
	}

	// Save new mode
	eRxMode = eMode;

	// Send command to radio
	SetMode(eMode);

	lRxStepSize = GetStepSize(eMode);

	return;
}

// end kenwood.cpp
