/****************************************************************************
Date: 9 SEPT 2001
File: ts2000.cpp
Description:
	ts2000 radio driver

History:
	13 APR 2001 G6LVB
	9 SEPT 2001 AA2TX - change to fix internal compiler error

*****************************************************************************/
#include "TS2000.hpp"
#include <string.h>

// Set Mode Table
// Index into table is MODULATION type, returns TS-2000 mode code
const CHAR TS2000::aSetModeTable[] = {
	'0',	// NA
	'1',	// LSB	
	'2',	// USB
	'3',	// CW
	'4',	// FM
	'5',	// AM
	'6'		// FSK
};

// Get Mode Table
// index into table is Kenwood mode code, returns MODULATION value
const MODULATION TS2000::aGetModeTable[] = {
		MODULATION_NA,	// 0 invalid code
		MODULATION_LSB, // 1
		MODULATION_USB, // 2
		MODULATION_CW,	// 3
		MODULATION_FM,	// 4
		MODULATION_AM,	// 5
		MODULATION_FSK,	// 6
		MODULATION_CW,	// 7 CE reverse
		MODULATION_NA,	// 8 invalid code
		MODULATION_FSK,	// 9 FSK reverse
		MODULATION_NA,	// 10 invalid code
		MODULATION_NA,	// 11 invalid code
		MODULATION_NA,	// 12 invalid code
		MODULATION_NA,	// 13 invalid code
		MODULATION_NA,	// 14 invalid code
		MODULATION_NA	// 15 invalid code

};


// Send a message to the radio
VOID	TS2000::SendMessage(void) {
	strcat(aOutputBuffer,TS2000_ENDOFMESSAGE);
	Comport.Write(aOutputBuffer, strlen(aOutputBuffer));
	eMessage = TS2000_MESSAGE_NONE;
	unMessageSize=0;
}

VOID	TS2000::SendMessage(const CHAR *aMessage)	{
	strcpy(aOutputBuffer,aMessage);
	SendMessage();
}

VOID	TS2000::SendMessage(UINT unSize) {
	SendMessage();
	unMessageSize=unSize;
}

VOID	TS2000::SendMessage(const CHAR *aMessage, UINT unSize) {
	SendMessage(aMessage);
	unMessageSize=unSize;
}

/****************************************************************************
Function: SetFrequency()				PRIVATE
Description: Set radio to this frequency
Parameters:
	Steps = desired frequency in steps
	ucOpcode = opcode to use in message
Returns:
*****************************************************************************/
VOID	TS2000::SetFrequency(LONG	lFrequencySteps,
			    const CHAR *pszSetVFOCmd) {

	CHAR* pBuffer=aOutputBuffer;

	while (( *pBuffer = *pszSetVFOCmd ) != '\0') {
		pBuffer++;
		pszSetVFOCmd++;
	}

	*pBuffer++ = '0'; // Write leading 0 (10GHz).

	// Convert Steps to frequency
	LONG	lFrequency = lFrequencySteps * TS2000_STEPSIZE;

	LONG lDivisor = TS2000_1GIGAHERTZ;

	for (INT i=0; i<10; i++) {
		LONG lDigit = lFrequency / lDivisor;
		*pBuffer++ = (CHAR) (lDigit + '0');
		lFrequency -= (lDigit * lDivisor);
		lDivisor /= 10;
	}

	*pBuffer = '\0';

	SendMessage();
	return;
}

/****************************************************************************
Function: ProcessVFO()				PRIVATE
Description: Decode VFO message from radio
	     Received message from radio is in aInputBuffer[]
Return Parameters:
	lFreqSteps = returned frequency steps
	emode	   = returned modulation type

Returns:
*****************************************************************************/
VOID	TS2000::ProcessVFO(LONG& lFreqSteps, MODULATION& eMode) {

			LONG lFreq = 0;		// frequency accumulator
			LONG lMultiplier = TS2000_1GIGAHERTZ;

			for ( INT i = 3; i <= 12; i++) {
				lFreq += (aInputBuffer[i] - '0') * lMultiplier;
				lMultiplier /= 10;
			}

			// Next, get modulation and convert
			// to RADIO::MODULATION enum type
			// byte 4 = modulation code

			// Not actually used in TX VFO, and result is trash as mode
			// not fetched on TX VFO
		
			eMode = aGetModeTable[(aInputBuffer[16] - '0') & 0x0F];

			// Calculate freq steps
			lFreqSteps = (lFreq + (TS2000_STEPSIZE/2)) /
							TS2000_STEPSIZE;

			// Return params
			// lFreqSteps = frequency steps
			// eMode = modulation

			return;
}

/****************************************************************************
Function: TS2000()
Description: constructor
Parameters:
	RADIOPARAMS structure
Returns:
*****************************************************************************/
TS2000::TS2000(const RADIOPARAMS& Params) :
	
	// Initialize
	RADIO(),
	eReady(BOOL_TRUE),
	eMessage(TS2000_MESSAGE_NONE),
	lRxVFOSteps(0),
	eRxMode(MODULATION_NA),
	eUserHasTunedRx(BOOL_FALSE),
	lTxVFOSteps(0),
	lTxXITSteps(0),
	eTxMode(MODULATION_NA),
	eCheckBand(BOOL_TRUE),
	unByteCounter(0) {

	//********************************************************
	// Open com port and check communications with radio
	// If fails, return with Operational flag FALSE
	//********************************************************
	RADIO::eOperational = Comport.Open(
					Params.unPort,
					Params.unRate ?
						Params.unRate :
						TS2000_BITRATE,
					TS2000_DATABITS,
					TS2000_PARITY,
					TS2000_STOPBITS,
					Params.unIRQ,
					Params.Address);

	if (!RADIO::eOperational) {
		return;		// failed to open
	}

	//**********************************************************
	// Check communications with Radio.
	// Send test message to Radio and start a 1 second timer.
	// If timer expires before we receive test message response,
	// communications has failed.
	//***********************************************************

	// See if radio is talking
	SendMessage(TS2000_TEST);
	USHORT		unCounter = 0;
	Timer.Start(1);
	while(unCounter < TS2000_TESTSIZE) { 
		if (Timer.IsExpired()) {
 			// Radio failed to communicate
			RADIO::eOperational = BOOL_FALSE;
			return;
		}
		if (Comport.Getc() != COMDRV_EOF) {
			unCounter++;
		} 
	} // Radio is talking!

	Timer.Stop();

	// if satellite (full duplex) mode
	if (Params.eTransceive) {

		eSatMode = BOOL_TRUE;
		SendMessage(TS2000_SATMODEON);
		unCounter=0;

		Timer.Start(1);
		while(unCounter < TS2000_SATMODEONSIZE) { 
			if (Timer.IsExpired()) {
 				// Radio failed to communicate
				RADIO::eOperational = BOOL_FALSE;
				return;
			}

			INT nByte = Comport.Getc();
			if (nByte != COMDRV_EOF) {
				// Got a byte, save it
				aInputBuffer[unCounter++] = (UCHAR) nByte;
			} 
		} // Radio is talking!

		Timer.Stop();		// stop message timeout timer

		// Now to find out what the bands are set to...
/******* Change due to compiler error in C7, 9/9/2001 AA2TX
		DOUBLE dRxFreqInit = 0.0;
		DOUBLE dTxFreqInit = 0.0;
		for (unCounter = 0 ; unCounter < 11 ; unCounter++)
		{
			dRxFreqInit = dRxFreqInit * 10.0 + aInputBuffer[2 + unCounter] - '0';
			dTxFreqInit = dTxFreqInit * 10.0 + aInputBuffer[16 + unCounter] - '0';
		}
*********/
		// ***** Try this instead - AA2TX SEPT 9. 2001
		// terminate freq strings to use atol()
		aInputBuffer[13] = 0;
		aInputBuffer[27] = 0;
		DOUBLE dRxFreqInit = (DOUBLE) atol((CHAR*) &aInputBuffer[2]);
		DOUBLE dTxFreqInit = (DOUBLE) atol((CHAR*) &aInputBuffer[16]);
		// End - AA2TX

		eRxBandInit=FreqToBand(dRxFreqInit);
		eTxBandInit=FreqToBand(dTxFreqInit);
	}
	else {
		eSatMode = BOOL_FALSE;
		SendMessage(TS2000_SATMODEOFF);
	}

	return;
}

/****************************************************************************
Function: 	~TS2000()
Description:	destructor
Parameters:
Returns:
*****************************************************************************/
TS2000::~TS2000() {

	// Turn off SAT mode
	SendMessage(TS2000_SATMODEOFF);

	// Wait for message to be sent
	Timer.Start(1);
	while (!Timer.IsExpired()) { }

	// Clean-up
	Comport.Close();
	return;
}

/****************************************************************************
Function: Update()
Description:
		Processes messages received from radio
Parameters:
	
Returns:
*****************************************************************************/
VOID	TS2000::Update() {

	// if radio is ready, nothing to do
	if (eReady) {
		return;
	}

	// Check for next message byte received
	INT nByte = Comport.Getc();
	if (nByte != COMDRV_EOF) {

		// Got a byte, save it
		aInputBuffer[unByteCounter++] = (UCHAR) nByte;

		// if last byte in message, process it
		if ( unByteCounter >= unMessageSize ) {

			Timer.Stop();		// stop message timeout timer
			unByteCounter = 0;	// reset 

			LONG		lFreqSteps;
			MODULATION	eMode;
			
			ProcessVFO(lFreqSteps, eMode);
			if (eMessage == TS2000_MESSAGE_RXVFO) {

				// See if user has manually tuned radio
				// if so, save new freq steps
				if (lFreqSteps != lRxVFOSteps) {
					eUserHasTunedRx = BOOL_TRUE;
					lRxVFOSteps = lFreqSteps;
				}
	
				// Save mode
				eRxMode = eMode;

				if (eSatMode) {
					// send get TxVFO message
					SendMessage(TS2000_VFOB,TS2000_VFOBSIZE);
					eMessage = TS2000_MESSAGE_TXVFO;

					Timer.Start(1);
				}
				else {	// done
					eMessage = TS2000_MESSAGE_NONE;
					eReady = BOOL_TRUE;
				}
			} 
			else { // TS2000_MESSAGE_TXVFO received

				// if user has changed sub band tuning
				// add in new XIT steps
				lTxXITSteps += (lFreqSteps - lTxVFOSteps);
		
				eMessage = TS2000_MESSAGE_NONE;
				eReady = BOOL_TRUE;
			}

		} // end of process received message
	}

	// Check for message timeout
	if (Timer.IsExpired()) {

		// if timed out, try again
		RefreshRxParams();
	}

	return;
}

/****************************************************************************
Function: SetTxFreq()
Description:
	Set transmit frequency if different from current frequency
Parameters:
	Frequency
Returns:
*****************************************************************************/
VOID	TS2000::SetTxFreq(DOUBLE dFrequency) {

	// Calculate new frequency steps 
	LONG lNewTxVFOSteps = FreqtoSteps(dFrequency);

	// Adjust for sub band tuning (XIT)
	lNewTxVFOSteps += lTxXITSteps;

	// Check to see if freq steps has changed
	// if so, update VFO steps and send new freq to radio
	if (lTxVFOSteps != lNewTxVFOSteps) {
		lTxVFOSteps = lNewTxVFOSteps;
		SetFrequency(lNewTxVFOSteps, eSatMode ? TS2000_VFOB : TS2000_VFOA);
	}
	return;
}

/****************************************************************************
Function: SetTxMode()
Description:
	Set transmitter modulation mode type
Parameters:
	Mode
Returns:
*****************************************************************************/
VOID	TS2000::SetTxMode(MODULATION eMode) {

	// Send message if different from current tx mode
	if (eTxMode != eMode) {
		CHAR szBuffer[TS2000_MAXMESSAGE];
		CHAR szMode[2];

		szMode[0]=aSetModeTable[eMode];
		szMode[1]='\0';

		strcpy(szBuffer, eSatMode ? TS2000_SATONTXMODEPRE : TS2000_SATOFFTXMODEPRE);
		strcat(szBuffer,szMode);
		strcat(szBuffer, eSatMode ? TS2000_SATONTXMODEPOST : TS2000_SATOFFTXMODEPOST);

		SendMessage(szBuffer);

		eTxMode = eMode;	// Save new tx mode
	}
	return;
}

/****************************************************************************
Function: RefreshRxParams()
Description:
Parameters:
Returns:
*****************************************************************************/
VOID	TS2000::RefreshRxParams() {

	eReady = BOOL_FALSE;
	SendMessage(TS2000_STATUS,TS2000_STATUSSIZE);
	eMessage = TS2000_MESSAGE_RXVFO;	// sending get rx vfo message
	Timer.Start(1);			// 1 second timeout timer
	return;
}

/****************************************************************************
Function: GetRxFreq()
Description:
	Gets Receive (VFO) frequency
Parameters:
Returns:
	Frequency
*****************************************************************************/
DOUBLE	TS2000::GetRxFreq() {
	eUserHasTunedRx = BOOL_FALSE;
	return (DOUBLE) (lRxVFOSteps * TS2000_STEPSIZE);
}

/****************************************************************************
Function: SetRxFreq()
Description:
	Set Receive frequency
Parameters:
	Frequency
Returns:
*****************************************************************************/
VOID	TS2000::SetRxFreq(DOUBLE dFrequency) {

	eUserHasTunedRx = BOOL_FALSE;

	if (eCheckBand && eSatMode) {	// force to make sure we can set bands
		eCheckBand = BOOL_FALSE;

		TS2000_BAND eRxBandReq=FreqToBand(dFrequency); // Get the requested band

		if (eRxBandReq == eTxBandInit) { // If req RX band == init TX band, we need to be smart...
			DOUBLE dTxFreqTemp;

			if ((eRxBandInit == TS2000_BAND_H && eTxBandInit == TS2000_BAND_V) ||
				(eRxBandInit == TS2000_BAND_V && eTxBandInit == TS2000_BAND_H)) {
				dTxFreqTemp = 436000000.0;
			}
			else {
				if ((eRxBandInit == TS2000_BAND_H && eTxBandInit == TS2000_BAND_U) ||
					(eRxBandInit == TS2000_BAND_H && eTxBandInit == TS2000_BAND_L) ||
					(eRxBandInit == TS2000_BAND_U && eTxBandInit == TS2000_BAND_H) ||
					(eRxBandInit == TS2000_BAND_L && eTxBandInit == TS2000_BAND_H)) {
					dTxFreqTemp = 145900000.0;
				}
				else {
					dTxFreqTemp = 29500000.0;
				}
			}
			SetFrequency(FreqtoSteps(dTxFreqTemp), TS2000_VFOB); // force TX band to safe band
		}
	}

	LONG lNewRxVFOSteps = FreqtoSteps(dFrequency);
	if (lRxVFOSteps != lNewRxVFOSteps) {
		lRxVFOSteps = lNewRxVFOSteps;
		SetFrequency(lNewRxVFOSteps, TS2000_VFOA);
	}

	return;
}

/****************************************************************************
Function: SetRxMode()
Description:
	Set receive modulation mode type.
Parameters:
	Mode
Returns:
*****************************************************************************/
VOID	TS2000::SetRxMode(MODULATION eMode) {

	// Send message if new mode is different from current mode
	if (eRxMode != eMode) {
		CHAR szBuffer[TS2000_MAXMESSAGE];
		CHAR szMode[2];

		szMode[0]=aSetModeTable[eMode];
		szMode[1]='\0';

		strcpy(szBuffer, TS2000_MODE);
		strcat(szBuffer,szMode);

		SendMessage(szBuffer);

		eRxMode = eMode; 	// Save new Rx mode
	}
	return;
}

// end TS2000.cpp
