/****************************************************************************
Date: 9/9/2001
File: ts790.hpp
Description:
	KENWOOD TS-790 radio driver

History:
	6/14/2001 Initial Version
	6/21/2001 B version
	7/3/2001 improved SUB tuning
	7/28/2001 add RIT to SUB
	8/1/2001  RIT didn't work, add XIT on MAIN instead
	8/2/2001  Fixed bug in Update() IF decode logic
	8/4/2001  Change Tx tuning steps to 100Hz
	8/5/2001  make MAIN update wait for 1 second
	8/17/2001 delay setting SUB until MAIN is set (init problem)
	8/18/2001 take out 1 second MAIN update delay
	9/2/2001  Allow Tx step size to be programmable in SSB/CW
	9/9/2001  Add in RX and TX on MAIN modes
*****************************************************************************/
#ifndef TS790_HPP
#define TS790_HPP

#include "loctypes.h"
#include "radio.hpp"	// RADIO base class
#include "slink.hpp"	// SERIALLINK class (includes UARTPARITY definition)

//***** Frequency Steps ****************
const LONG TS790_STEP_SSB	= 20;	// freq step Hz in SSB/CW modes (rx)
const LONG TS790_STEP_FM	= 100;	// freq step Hz when FM (rx)
const LONG TS790_1GIGAHERTZ 	= 1000000000L; // 1GHz

//***** Message Definitions ************
const INT  TS790_MAXMESSAGE	= 40;   // max number of chars in a message
const INT  TS790_OPCODESIZE	= 2;   // all opcodes are 2 chars
const CHAR TS790_ENDOFMESSAGE	= ';'; // End of message delimiter

//****** OpCodes ***********************
// These are used to build up commands
const CHAR* const TS790_MODE	= "MD";	// set mode (+ 1 digit)
const CHAR* const TS790_VFOA	= "FA"; // Set or read VFO A
const CHAR* const TS790_VFOB	= "FB"; // Set or read VFO B

//***** Command Messages ****************
// These are complete commands sent to the radio
const CHAR* const TS790_MAINBAND	= "DC0"; // destination main band
const CHAR* const TS790_SUBBAND		= "DC1"; // destination sub band
const CHAR* const TS790_INFO		= "IF";	// read info
const CHAR* const TS790_SELECTVFOA	= "FN0"; // function VFO A
const CHAR* const TS790_SELECTVFOB	= "FN1"; // function VFO B
const CHAR* const TS790_STEPOFF		= "ST0"; // Step tuning off
const CHAR* const TS790_LOCKON		= "LK1"; // lock front panel
const CHAR* const TS790_LOCKOFF		= "LK0"; // unlock front panel
const CHAR* const TS790_TESTMESSAGE	= " ";  // used to see if radio is ok


//***** Response Messages *****************
// These are used to decode response messages
const INT  TS790_MSGIF_CODE	= ((INT) 'I' + 'F');	// IF message
const INT  TS790_MSGIF_FREQOFFSET = 2;	// IF message, frequency
const INT  TS790_MSGIF_STEPOFFSET = 13;	// IF message, step offset
const INT  TS790_MSGIF_RITFREQOFFSET = 18;  // IF message, RIT frequency
const INT  TS790_MSGIF_RITOFFSET = 23;  // IF message, RIT on/off
const INT  TS790_MSGIF_MODEOFFSET = 29;	// IF message, Mode offset
const CHAR TS790_MSGIF_ON	    = '1'; // IF message param = ON
const CHAR TS790_MSGIF_OFF	    = '0'; // IF message param = OFF
const CHAR TS790_MSGIF_NONE	    = ' '; // IF message param missing

//***** COM PORT PARAMETERS ******************************************
const USHORT	TS790_TIMEOUT  = 2;	// 2 second response timer
const UINT	TS790_BITRATE  = 4800;
const UINT	TS790_DATABITS = 8;
const UINT	TS790_STOPBITS = 2;
const UARTPARITY TS790_PARITY	 = UARTPARITY_NONE;


class TS790 : public RADIO {
private:
	BOOL		eXCVR;		// TRUE if used for both TX and RX
	BOOL		eReady;		// ready for next command
	BOOL		eHoldTuning;	// Stop tuning - used for XIT function
	BOOL		eMainSet;	// Has MAIN been initialized?
					// used to fix init problem on SUB
	DOUBLE		dSaveFrequency;	// used to remember SUB freq for init

	// Rx Params
	const CHAR*	pszRXVFO;	// RX VFO command string
	LONG		lRxStepSize;	// frequency step size
	LONG		lRxSteps;	// frequency steps
	MODULATION	eRxMode;	// emission
	BOOL		eUserHasTunedRx; // TRUE if user turned dial

	// Tx Params
	LONG		lTxSSBStepSize;	// step size when in SSB/CW modes
	LONG		lTxStepSize;	// current frequency step size
	LONG		lTxVFOSteps;	// frequency steps of "virtual" VFO
	LONG		lTxXITSteps;	// Tx independent tuning offset (XIT)
	LONG		lTxActualSteps;	// actual radio frequency steps
	MODULATION	eTxMode;	// current Tx emission type
	
	// Mode tables
	// Converts between MODULATION type and kenwood message params
	static const MODULATION aGetModeTable[]; // Kenwood to MODULATION
	static const CHAR aSetModeTable[];	 // MODULATION to Kenwood

	// Serial I/O
	SERIALLINK	SerialLink;
	CHAR		aInputBuffer[TS790_MAXMESSAGE]; // from radio
	CHAR		aOutputBuffer[TS790_MAXMESSAGE]; // to radio

	//******************************************************
	// Utilities
	//******************************************************

	// Frequency to Frequency Steps
	LONG	FreqtoSteps(DOUBLE dFrequency, LONG lStepSize) {
		// ***** DO NOT CHANGE *****
		// workaround for MSC7 DOUBLE conversion bug
		volatile ULONG ulFrequency = (ULONG) dFrequency;
		return ( ((LONG) ulFrequency) + (lStepSize/2) ) / lStepSize;
	}

	// Set radio frequency
	VOID SetFrequency(LONG	lNewSteps,
			  LONG 	lStepSize,
			  const CHAR* pszSetVFOCmd);

	// Set radio mode
	VOID	SetMode(MODULATION eMode);

	// Get Frequency Step Size for this mode
	LONG	GetRxStepSize(MODULATION eMode);
	LONG	GetTxStepSize(MODULATION eMode);

	// Select BAND (MAIN or SUB) 
	VOID	SelectTXBand();
	VOID	SelectRXBand();


private:
	// No copy constructor or assignment allowed
	// Force compile error if attempted
	TS790(const TS790&);
	TS790& operator=(const TS790&);

public:
	// General Functions
	TS790(const RADIOPARAMS& Params);
	BOOL	IsReady() { return eReady; }
	VOID	Update();
	~TS790();

	// Tx Functions
	VOID	SetTxFreq(DOUBLE dFrequency);
	VOID	SetTxMode(MODULATION eMode);

	// Rx Functions
	BOOL	HasUserTunedRx() { return eUserHasTunedRx; }
	VOID	RefreshRxParams();
	DOUBLE	GetRxFreq();
	MODULATION GetRxMode() { return eRxMode; }
	VOID	SetRxFreq(DOUBLE dFrequency);
	VOID	SetRxMode(MODULATION eMode);

};
#endif // end ts790.hpp
